@@ -1,4 +1,3 @@
-crate::use_native_or_external!(List);
 use crate::lexer::*;
 use crate::Bytes;
 use crate::TokenBuf;
@@ -8,7 +7,7 @@ impl Lexer {
         if cfg!(feature = "debug-lexer") {
             println!("set_yylval_id({})", id);
         }
-        self.lval = Some(Bytes::new(List::from(id)));
+        self.lval = Some(Bytes::new(Vec::from(id)));
     }
 
     pub(crate) fn set_yylval_literal(&mut self, value: &TokenBuf) {
@@ -25,7 +24,7 @@ impl Lexer {
         if cfg!(feature = "debug-lexer") {
             println!("set_yylval_num {:#?}", flags);
         }
-        self.lval = Some(Bytes::new(List::from(flags)));
+        self.lval = Some(Bytes::new(Vec::from(flags)));
     }
 
     pub(crate) fn set_yylval_str(&mut self, value: &TokenBuf) {
