@@ -4,6 +4,8 @@
 #![warn(unused_qualifications)]
 #![warn(deprecated_in_future)]
 #![warn(unused_lifetimes)]
+#![allow(clippy::boxed_local)]
+#![forbid(unsafe_code)]
 #![doc = include_str!("../README.md")]
 
 /*!
@@ -12,6 +14,8 @@ A Ruby parser written in Rust.
 Uses bison under the hood.
 */
 
+extern crate alloc_from_pool;
+
 mod loc;
 pub use loc::Loc;
 
@@ -88,13 +92,5 @@ pub use token::Token;
 mod bytes;
 pub use bytes::Bytes;
 
-/// Module with generic containers
-pub mod containers;
-pub(crate) use containers::use_native_or_external;
-
-/// Module with blobs, based on provided sizes
-#[cfg(feature = "compile-with-external-structures")]
-pub mod blobs;
-
 #[cfg(test)]
 pub(crate) mod test_helpers;
